/* This source file is public domain (do whatever you wish with it).
   Created by Heikki Orsila <heikki.orsila@tut.fi>
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "btree.h"

tnode *btree_createtree(void *data, tnodekey *key) {
  tnode *new = calloc(1,sizeof(tnode));
  new->data = data;
  new->key = *key;
  return new;
}

void btree_removetree(tnode *node) {
  if(node) {
    if(node->low) btree_removetree(node->low);
    if(node->high) btree_removetree(node->high);
    free(node);
    node = 0;
  }
}

tnode *btree_addnode(tnode *root, void *data, tnodekey *key, int insert) {
  tnode *node = root, *parent = 0, *newnode;
  tnodekey newkey = *key;
  int or = 0;

  if(node==0) return 0;

  while(node) {
    or = strcasecmp(newkey.key,node->key.key);
    parent = node;
    if(or < 0) {
      node = node->low;
    } else if(or > 0) {
      node = node->high;
    } else {
      return ((insert) ? 0:node);
    }
  }
  newnode = 0;
  if(insert) {
    newnode = calloc(1,sizeof(tnode));
    if(or<0)
      parent->low = newnode;
    else
      parent->high = newnode;
    newnode->key = newkey;
    newnode->data = data;
  }
  return ((insert) ? newnode : 0);
}

int btree_traverse(tnode *node, int (*callback)(tnode *node, void *arg), void *arg) {
  if(node) {
    if(node->low) {
      if(btree_traverse(node->low,callback,arg) == 0) return 0;
    }
    if(callback(node,arg) == 0) return 0;
    if(node->high) {
      if(btree_traverse(node->high,callback,arg) == 0) return 0;
    }
  }
  return 1;
}

int btree_removenode(tnode **root, tnodekey *key) {
  tnode *node = *root, *parent = 0, *repl, *copy;
  int or=0,copyor;
  if(node==0) return 0;
  while(node) {
    or = strcasecmp(key->key,node->key.key);
    if(or<0) {
      parent = node;
      node = node->low;
    } else if(or>0) {
      parent = node;
      node = node->high;
    } else {
      break;
    }
  }
  if(node==0) return 0;

  if(node->low!=0) {
    repl = node->low;
    copy = node->high;
    copyor = 1;
  } else if(node->high!=0) {
    repl = node->high;
    copy = node->low;
    copyor = -1;
  } else {
    repl = 0;
    copy = 0;
    copyor = -1;
  }

  /* parent=0 if node to be removed is root */
  if(parent) {
    if(parent->low == node) {
      parent->low = repl;
    } else {
      parent->high = repl;
    }
  } else {
    /* node to remove is root */
    *root = repl;
  }

  free(node);
  node = 0;

  if(repl==0) return 1;

  node=repl;
  while(1) {
    if(copyor<0) {
      if(node->low == 0) {
	node->low = copy;
	break;
      }
      node = node->low;
    } else {
      if(node->high == 0) {
	node->high = copy;
	break;
      }
      node = node->high;
    }
  }
  return 1;
}
