 /* 
  * UAE - The Un*x Amiga Emulator
  * 
  * Support for Linux/USS sound
  * 
  * Copyright 1997 Bernd Schmidt
  */

#include "sysconfig.h"
#include "sysdeps.h"

#include "config.h"
#include "options.h"
#include "memory.h"
#include "custom.h"
#include "gensound.h"
#include "sd-sound.h"

#include <sys/ioctl.h>

#ifdef HAVE_SYS_SOUNDCARD_H
#include <sys/soundcard.h>
#elif defined HAVE_MACHINE_SOUNDCARD_H
#include <machine/soundcard.h>
#elif defined HAVE_SOUNDCARD_H
#include <soundcard.h>
#else
#error "Something went wrong during configuration."
#endif

#include "uade.h"


int sound_fd;
static int have_sound = 0;
static unsigned long formats;

uae_u16 sndbuffer[44100];
uae_u16 *sndbufpt;
int sndbufsize;

static int exact_log2 (int v)
{
    int l = 0;
    while ((v >>= 1) != 0)
	l++;
    return l;
}

void close_sound (void)
{
    if (have_sound)
	close (sound_fd);
}

/* Try to determine whether sound is available.  This is only for GUI purposes.  */
int setup_sound (void) {
  if(!uade_no_native_sound) {
    sound_available = 0;
    sound_fd = open ("/dev/dsp", O_WRONLY);
    if(sound_fd < 0) {
      perror ("Can't open /dev/dsp");
      if (errno == EBUSY) {
	/* We can hope, can't we ;) */
	sound_available = 1;
	return 1;
      }
      return 0;
    }
    if(ioctl (sound_fd, SNDCTL_DSP_GETFMTS, &formats) == -1) {
      perror ("ioctl failed - can't use sound");
      close (sound_fd);
      return 0;
    }
    close(sound_fd);
  }
  sound_available = 1;
  return 1;
}

int init_sound (void) {
  int tmp;
  int rate;
  int dspbits;
  
  if (currprefs.sound_maxbsiz < 128 || currprefs.sound_maxbsiz > 16384) {
    fprintf (stderr, "Sound buffer size %d out of range.\n", currprefs.sound_maxbsiz);
    currprefs.sound_maxbsiz = 8192;
  }

  if(!uade_no_native_sound) {
    sound_fd = open ("/dev/dsp", O_WRONLY);
    have_sound = !(sound_fd < 0);
    if (! have_sound) {
      perror ("Can't open /dev/dsp");
      if (errno != EBUSY)
	sound_available = 0;
      return 0;
    }
    if (ioctl (sound_fd, SNDCTL_DSP_GETFMTS, &formats) == -1) {
      perror ("ioctl failed - can't use sound");
      close (sound_fd);
      have_sound = 0;
      return 0;
    }
    // this was 0x40000 in uae 0.8.9
    tmp = 0x00040000 + exact_log2 (currprefs.sound_maxbsiz);
    ioctl (sound_fd, SNDCTL_DSP_SETFRAGMENT, &tmp);
    // this gets sndbufsize !
    ioctl (sound_fd, SNDCTL_DSP_GETBLKSIZE, &sndbufsize);
  } else {
    have_sound = 1;
    formats = AFMT_S16_LE;
    sndbufsize = 4096;
  }
  
  dspbits = currprefs.sound_bits;
  tmp     = currprefs.stereo;
  rate    = currprefs.sound_freq;

  if(!uade_no_native_sound) {
    ioctl (sound_fd, SNDCTL_DSP_SAMPLESIZE, &dspbits);
    ioctl (sound_fd, SOUND_PCM_READ_BITS, &dspbits);
    if(dspbits != currprefs.sound_bits) {
      fprintf (stderr, "Can't use sound with %d bits\n", currprefs.sound_bits);
      return 0;
    }
    ioctl (sound_fd, SNDCTL_DSP_STEREO, &tmp);
    ioctl (sound_fd, SNDCTL_DSP_SPEED, &rate);
    ioctl (sound_fd, SOUND_PCM_READ_RATE, &rate);
    /* Some soundcards have a bit of tolerance here. */
    if (rate < currprefs.sound_freq * 90 / 100 || rate > currprefs.sound_freq * 110 / 100) {
      fprintf (stderr, "Can't use sound with desired frequency %d\n", currprefs.sound_freq);
      return 0;
    }
  }

  sample_evtime = (long)maxhpos * maxvpos * 50 / rate;
  if (dspbits == 16) {
    /* Will this break horribly on bigendian machines? Possible... */
    if (!(formats & AFMT_S16_LE)) {
      fprintf(stderr, "init_sound: dspbits error...\n");
      return 0;
    }
    init_sound_table16 ();
    sample_handler = currprefs.stereo ? sample16s_handler : sample16_handler;
  } else {
    if (!(formats & AFMT_U8))
      return 0;
    init_sound_table8 ();
    sample_handler = currprefs.stereo ? sample8s_handler : sample8_handler;
  }
  sound_available = 1;
  /*
    if(!uade_no_native_sound) {
    fprintf (stderr,"Sound driver found and configured for %d bits at %d Hz, buffer is %d bytes\n", dspbits, rate, sndbufsize);
    }
  */
  sndbufpt = sndbuffer;
#ifdef FRAME_RATE_HACK
  vsynctime = vsynctime * 9 / 10;
#endif	
  return 1;
}
